import {
  schemeTableau10,
  scaleOrdinal,
  area,
  pie,
  arc,
} from 'd3';
import { select } from 'd3-selection';

export const pieChart = (
  selection,
  { data, height, width }
) => {
  const radius = Math.min(width, height) / 2 - 10;

  const svg = selection
    .append('svg')
    .attr('width', width)
    .attr('height', height);

  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${width / 2}, ${height / 2})`
    );

  const color = scaleOrdinal()
    .domain(data.map((d) => d.classification))
    .range(schemeTableau10);

  const legend = svg
    .selectAll('.legend')
    .data(color.domain())
    .enter()
    .append('g')
    .attr('class', 'legend')
    .attr(
      'transform',
      (d, i) => `translate(0,${i * 20 + 10})`
    )
    .on('mouseover', function (event, d) {
      pieArc
        .filter(
          (arcData) =>
            arcData.data.classification !== d
        )
        .transition()
        .duration(200)
        .attr('opacity', 0.2);
    })
    .on('mouseout', function (event, d) {
      pieArc
        .transition()
        .duration(200)
        .attr('opacity', 0.8);
    });

  legend
    .append('rect')
    .attr('x', width - 28)
    .attr('width', 18)
    .attr('height', 18)
    .style('fill', color)
    .attr('opacity', 0.8);

  legend
    .append('text')
    .attr('x', width - 34)
    .attr('y', 9)
    .attr('dy', '.35em')
    .style('font-family', 'sans-serif')
    .style('text-anchor', 'end')
    .text((d) => d);

  const pieData = pie().value((d) => d.count)(
    data
  );

  const arcPath = arc()
    .outerRadius(radius)
    .innerRadius(0);

  const pieArc = g
    .selectAll('path')
    .data(pieData)
    .enter()
    .append('path')
    .attr('d', arcPath)
    .attr('fill', (d, i) => schemeTableau10[i])
    .attr('stroke', '#fff')
    .attr('stroke-width', '2px')
    .attr('opacity', 0.8)
    .on('mouseover', function (event, d) {
      select(this).attr('opacity', 1);
      const [x, y] = arcPath.centroid(d);
      const tooltip = g
        .append('g')
        .attr('class', 'tooltip')
        .attr('transform', `translate(${x},${y})`)
        .style('opacity', 0)
        .style('pointer-events', 'none');
      tooltip
        .append('rect')
        .attr('width', 200)
        .attr('height', 50)
        .attr('fill', 'white')
        .attr('rx', 5)
        .style('stroke', 'black')
        .style('stroke-width', 0.5);
      tooltip
        .append('text')
        .attr('x', 100)
        .attr('y', 27)
        .text(
          `${d.data.classification}: ${d.data.count}`
        )
        .style('font-size', '14px')
        .style('font-family', 'sans-serif')
        .style('fill', 'black')
        .style('text-anchor', 'middle');
      tooltip
        .transition()
        .duration(200)
        .style('opacity', 1);
    })
    .on('mousemove', function (event, d) {
      const [x, y] = arcPath.centroid(d);
      const tooltip = select(this).select(
        '.tooltip'
      );
      tooltip.attr(
        'transform',
        `translate(${x},${y})`
      );
    })
    .on('mouseout', function (event, d) {
      select(this).attr('opacity', 0.8);
      select('.tooltip').remove();
    });
};
